/*
 * Example for the usage of posix_memalign
 * Author: Daniel Flachs (dflachs@mail.uni-mannheim.de)
 *
 * int posix_memalign (void **memptr, size_t alignment, size_t size)
 * 
 * return:         0 -> success; else -> failure
 * 
 * parameters:
 *   - memptr:     address of a pointer to which the start address 
 *                 of the allocated memory is written
 *   - alignment:  memory is allocated such that *memptr mod alignment == 0, 
 *                 i.e., the size of the SIMD type in bytes (here 16).
 *                 alignment must be power of 2
 *   - size:       size of the allocated memory in bytes;
 *                 for 1000 4-byte integers, we need 1000 * sizeof(int)
 */

#include <cstdlib>
#include <cstddef>
#include <iostream>


int main() {
    constexpr size_t lSize = 1'000;  // number of elements in the array
    constexpr size_t lAlign = 16;    // alignment in bytes: SIMD -> 128 bits = 16 Bytes
    int* lMem = nullptr;

    // Allocate aligned memory
    if (0 == posix_memalign(reinterpret_cast<void**>(&lMem), lAlign, lSize*sizeof(int))) {
            std::cout << "malloc successful: " 
                << lMem << " mod " << lAlign << " = " << (reinterpret_cast<uint64_t>(lMem) % lAlign) 
                << std::endl;
    } else {
        std::cerr << "malloc failed. " << std::endl;
        return 1;
    }

    // Fill the memory with some dummy values and print them
    for (size_t i = 0; i < lSize; ++i) {
        lMem[i] = i;
        std::cout << lMem[i] << (i != lSize-1 ? ", " : ".");
    }
    std::cout << std::endl;

    // Free the memory and exit
    free(lMem);
    return 0;
}

